
/*
 * XKOBO, a video-oriented game
 * Copyright (C) 1995,1996  Akira Higuchi
 *     a-higuti@math.hokudai.ac.jp
 * 
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 675 Mass Ave, Cambridge, MA 02139, USA.
 * 
 */

extern "C"
{
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <unistd.h>
}
#include "skobo.h"
#include "screen.h"
#include "manage.h"
#include "scenes.h"
#include "score.h"
#include "enemies.h"
#include "myship.h"
#include "radar.h"
#include "key.h"
#include "sound/sounds.h"
#include "sound/bgmfile.h"

#define GIGA             1000000000
#define BONUS_FIRSTTIME  2000
#define BONUS_EVERY      3000

_manage_state _manage::state;
int _manage::next_state_out;
int _manage::next_state_next;
int _manage::highscore = 0;
int _manage::last_scene = 0;
int _manage::scene_num;
int _manage::ships;
int _manage::level;
int _manage::count;
int _manage::score;
int _manage::score_changed;
int _manage::bonus_next;
int _manage::flush_score_count = 0;
int _manage::flush_ships_count = 0;
int _manage::delay_count;
int _manage::rest_cores;
int _manage::return_value = 0;
static int sound_fine_playing = 0;

void _manage::game_start()
{
	ships = SHIPS;
	level = 0;
	score = 0;
	flush_score_count = 0;
	bonus_next = BONUS_FIRSTTIME;
	screen.init_scene(scene_num);
	_manage::init_resources_to_play();
}

void _manage::next_scene()
{
	scene_num++;
	if(scene_num >= GIGA - 1)
		scene_num = GIGA - 2;
	screen.init_scene(scene_num);
}

void _manage::retry()
{
	if(!cheat_mode)
		ships--;
	if(ships <= 0)
	{
		if(last_scene < scene_num)
			last_scene = scene_num;
		if(!cheat_mode)
		{
			if(highscore < score)
				highscore = score;
			scorefile.record(highscore, last_scene);
		}
		ships = 0;
		if(audio_is_active(SOUND_BGM))
			audio_stop(SOUND_BGM);
		_manage::init_resources_title();
		return;
	}
}

void _manage::init_resources_title()
{
	state = title;
	screen.init_scene(INTRO_SCENE);
	_manage::put_info();
}

void _manage::init_resources_to_play()
{
	state = playing;
	count = 0;
	delay_count = 0;
	flush_score_count = (flush_score_count) ? -1 : 0;
	flush_ships_count = 0;
	score_changed = 0;
	next_state_out = 0;
	next_state_next = 0;

	key.init();
	enemies.init();
	myship.init();
	rest_cores = screen.prepare();
	screen.generate_fixed_enemies();
	_manage::put_info();
	gengine.scroll(PIXEL2CS(myship.get_virtx()),
			PIXEL2CS(myship.get_virty()));
	gengine.force_scroll();
	if(audio_is_active(SOUND_BGM))
		audio_stop(SOUND_BGM);
	if(use_special_bgm)
		set_bgm(scene_num);
}

void _manage::put_info()
{
	static char s[16];

	snprintf(s, 16, "%09d", highscore);
	dhigh.text(s);
	dhigh.on();

	put_score();

	snprintf(s, 16, "%03d", scene_num + 1);
	dstage.text(s);
	dstage.on();

	if(!cheat_mode)
		snprintf(s, 16, "%03d", ships);
	else
		snprintf(s, 16, "999");
	dships.text(s);
	dships.on();
}

void _manage::put_score()
{
	static char s[1024];
	snprintf(s, 16, "%09d", score);
	dscore.text(s);
	dscore.on();
	score_changed = 0;
}

void _manage::flush_score()
{
	flush_score_count--;
	if(flush_score_count & 1)
		return;

	if(flush_score_count & 2)
		dscore.off();
	else
		dscore.on();
	if(flush_score_count == 0)
		flush_score_count = -1;
}

void _manage::flush_ships()
{
	flush_ships_count--;
	if(flush_ships_count & 1)
		return;

	if(flush_ships_count & 2)
		dships.off();
	else
		dships.on();
	if(flush_ships_count == 0)
		flush_ships_count = -1;
}

void _manage::eventloop()
{
#if 0
	while(win::xcheckevent())
	{
		winfo.eventloop();
		wchip.eventloop();
		wradar.eventloop();
		wscore.eventloop();
	}
#endif
}

/****************************************************************************/
void _manage::init()
{
	scorefile.init(&highscore, &last_scene);
	count = 0;
	ships = 0;
	level = 0;
	scene_num = last_scene;
	flush_ships_count = 0;
	delay_count = 0;
	screen.init();
	_manage::init_resources_title();
}

int _manage::mainloop()
{
	_manage::eventloop();

	switch(state)
	{
	  case title:
		static int demo_x = CHIP_SIZEX * (64-18);
		static int demo_y = CHIP_SIZEY * (64-7);
		gengine.scroll(PIXEL2CS(demo_x), PIXEL2CS(demo_y));
//		demo_x += 3;
		demo_y -= 3;
		demo_x &= MAP_SIZEX*CHIP_SIZEX-1;
		demo_y &= MAP_SIZEY*CHIP_SIZEY-1;
	  case paused:
		return return_value;
	}

	count++;
	if(delay_count)
		delay_count--;

	if(delay_count == 1)
	{
		if(enemies.exist_pipe())
		{
			delay_count = 2;
		}
		else
		{
			_manage::put_info();
			if(next_state_out)
			{
				_manage::retry();
				if(ships <= 0)
				{
					return return_value;
				}
			}
			if(next_state_next)
			{
				_manage::next_scene();
			}
			_manage::init_resources_to_play();
			return return_value;
		}
	}

	myship.move();
	enemies.move();
	myship.hit_structure();

	myship.put();
	enemies.put();
	if(score_changed)
		_manage::put_score();
	if(flush_score_count > 0)
		_manage::flush_score();
	if(flush_ships_count > 0)
		_manage::flush_ships();
	radar.trace_myship();
	gengine.scroll(PIXEL2CS(myship.get_virtx()),
			PIXEL2CS(myship.get_virty()));

	if(sound_fine_playing)
	{
		if(!audio_is_active(SOUND_FINE))
		{
			sound_fine_playing = 0;
			audio_volume(SOUND_BGM, DEFAULT_VOL);
		}
	}
	else if(!audio_is_active(SOUND_BGM) && rest_cores > 0)
	{
		if(use_special_bgm)
		{
			char *bgm;
			int n;

			bgm = next_bgm(&n);
			if(bgm != NULL)
				audio_play(SOUND_BGM, bgm, n);
			else
				play_sound_looped(SOUND_BGM);
		}
		else
			play_sound_looped(SOUND_BGM);
	}

	return return_value;
}

void _manage::lost_myship()
{
	if(delay_count == 0)
		delay_count = 20;
	next_state_out = 1;
}

void _manage::destroyed_a_core()
{
	play_sound(SOUND_EXPL3);
	audio_volume(SOUND_BGM, DEFAULT_VOL / 2);
	play_sound(SOUND_FINE);
	sound_fine_playing = 1;

	rest_cores--;
	if(rest_cores == 0)
	{
		next_state_next = 1;
		delay_count = 50;
	}
	screen.generate_fixed_enemies();
}

void _manage::start_key()
{
	switch (state)
	{
	  case title:
	  {
		_manage::game_start();
		break;
	  }
	  case paused:
	  {
		state = playing;
		break;
	  }
	  case playing:
	  {
		state = paused;
		break;
	  }
	}
}

void _manage::exit_key()
{
	if(state == title)
	{
		return_value = 1;
	}
	else
	{
		ships = 0;
		_manage::retry();
		_manage::put_info();
	}
}

void _manage::plus_key()
{
	if(state == title)
	{
		if((scene_num < last_scene) || cheat_mode)
			scene_num++;
		_manage::put_info();
		screen.init_scene(-scene_num-1);
	}
}

void _manage::minus_key()
{
	if(state == title)
	{
		scene_num--;
		if(scene_num <= 0)
			scene_num = 0;
		_manage::put_info();
		screen.init_scene(-scene_num-1);
	}
}

void _manage::add_score(int sc)
{
	score += sc;
	if(score >= GIGA)
		score = GIGA - 1;
	else if(!cheat_mode)
	{
		if(score >= bonus_next)
		{
			bonus_next += BONUS_EVERY;
			ships++;
			play_sound(SOUND_ONEUP);
			flush_ships_count = 50;
		}
		if((score >= highscore) && (flush_score_count == 0))
		{
			flush_score_count = 50;
		}
	}
	score_changed = 1;
}
